@@ -23,7 +23,7 @@
 #include "config.h"
 #include "gchemutils-priv.h"
 #include "gogcpapp.h"
-
+#include "gogcrystalapp.h"
 #include <gsf/gsf-impl-utils.h>
 #include <goffice/app/module-plugin-defs.h>
 #include <goffice/component/go-component-factory.h>
@@ -99,6 +99,24 @@ go_gchemutils_component_edit (GOComponent *component)
 	return gogcu->application->EditDocument (gogcu);
 }
 
+static void
+go_gchemutils_component_mime_type_set (GOComponent *component)
+{
+	if (!strcmp (component->mime_type, "application/x-gcrystal")) {
+		component->needs_window = true;
+#ifdef GO_SNAPSHOT_PNG
+		component->snapshot_type = GO_SNAPSHOT_PNG;
+#endif
+	}
+}
+
+static void
+go_gchemutils_component_set_window (GOComponent *component)
+{
+	if (!strcmp (component->mime_type, "application/x-gcrystal")) {
+	}
+}
+
 static void
 go_gchemutils_component_finalize (GObject *obj)
 {
@@ -117,6 +135,9 @@ go_gchemutils_component_init (GOComponent *component)
 	component->ascent = 1.;
 	component->descent = 0.;
 	component->width = 1.;
+#ifdef GO_SNAPSHOT_PNG
+	component->snapshot_type = GO_SNAPSHOT_SVG;
+#endif
 }
 
 static void
@@ -131,6 +152,8 @@ go_gchemutils_component_class_init (GOComponentClass *klass)
 	klass->set_data = go_gchemutils_component_set_data;
 	klass->render = go_gchemutils_component_render;
 	klass->edit = go_gchemutils_component_edit;
+	klass->mime_type_set = go_gchemutils_component_mime_type_set;
+	klass->set_window = go_gchemutils_component_set_window;
 }
 
 GSF_DYNAMIC_CLASS (GOGChemUtilsComponent, go_gchemutils_component,
@@ -148,7 +171,9 @@ go_plugin_init (GOPlugin *plugin, G_GNUC_UNUSED GOCmdContext *cc)
 	go_gchemutils_component_register_type (module);
 //	go_components_set_mime_suffix ("chemical/x-xyz", "*.xyz");
 	go_components_set_mime_suffix ("application/x-gchempaint", "*.gchempaint");
+	go_components_set_mime_suffix ("application/x-gcrystal", "*.gcrystal");
 	Apps["application/x-gchempaint"] = new GOGcpApplication ();
+	Apps["application/x-gcrystal"] = new GOGCrystalApplication ();
 //	go_components_set_mime_suffix ("application/x-gcrystal", "*.gcrystal");
 // TODO: add other types
 }
