@@ -57,28 +57,30 @@ gcu::Document *GOGcpApplication::ImportDocument (const string& mime_type, const
 	char *old_num_locale, *old_time_locale;
 	if (mime_type == "application/x-gchempaint") {
 		xmlDocPtr xml;
-		if (!(xml = xmlParseMemory(data, length)) ||
+		if (!(xml = xmlParseMemory (data, length)) ||
 			xml->children == NULL ||
-			strcmp((char*)xml->children->name, "chemistry"))
+			strcmp(reinterpret_cast <char const *> (xml->children->name), "chemistry")) {
+			xmlFreeDoc (xml);
 			return NULL;
+		}
 		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale(LC_NUMERIC, "C");
+		setlocale (LC_NUMERIC, "C");
 		old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
-		setlocale(LC_TIME, "C");
-		pDoc = new gcp::Document(this, false);
+		setlocale (LC_TIME, "C");
+		pDoc = new gcp::Document (this, false);
 		pDoc->GetView ()->CreateNewWidget ();
-		bool result = pDoc->Load(xml->children);
-		setlocale(LC_NUMERIC, old_num_locale);
-		g_free(old_num_locale);
-		setlocale(LC_TIME, old_time_locale);
-		g_free(old_time_locale);
-		if (!result)
-		{
+		bool result = pDoc->Load (xml->children);
+		setlocale (LC_NUMERIC, old_num_locale);
+		g_free (old_num_locale);
+		setlocale (LC_TIME, old_time_locale);
+		g_free (old_time_locale);
+		xmlFreeDoc (xml);
+		if (!result) {
 			delete pDoc;
 			return NULL;
 		}
 	} else {
-	}
+	}	
 	return (pDoc);
 }
 
@@ -159,12 +161,12 @@ void GOGcpApplication::UpdateBounds (GOGChemUtilsComponent *gogcu)
 	gccv::Rect rect;
 	gcp::WidgetData *pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	pData->GetObjectBounds (doc, &rect);
-	if (rect.x0 || rect.y0)
-		doc->Move (- rect.x0 / pTheme->GetZoomFactor (), - rect.y0 / pTheme->GetZoomFactor ());
-	doc->GetView ()->Update (doc);
 	double y = doc->GetYAlign ();
 	y += doc->GetView ()->GetBaseLineOffset ();
 	y *= pTheme->GetZoomFactor ();
+	if (rect.x0 || rect.y0)
+		doc->Move (- rect.x0 / pTheme->GetZoomFactor (), - rect.y0 / pTheme->GetZoomFactor ());
+	doc->GetView ()->Update (doc);
 	if (y < rect.y0)
 		y = rect.y1;
 	// assuming 96 dpi, setting dimensions as inches.
