@@ -42,10 +42,10 @@
 #include "window.h"
 #include <gcu/loader.h>
 #include <gcu/objprops.h>
+#include <gcu/xml-utils.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
-#include <clocale>
 #include <stack>
 #include <cstring>
 #include <sstream>
@@ -305,7 +305,6 @@ void Document::ExportOB () const
 	Atom* pgAtom;
 	unsigned index = 1;
 	double x, y, z;
-	gchar *old_num_locale;
 	map< string, Object * >::const_iterator m;
 	stack<map< string, Object * >::const_iterator> iters;
 	set<Object const *> Mols;
@@ -321,8 +320,6 @@ void Document::ExportOB () const
 			g_object_unref (file);
 			throw (int) 1;
 		}
-		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
 		OBConversion Conv;
 		OBFormat* pOutFormat = Conv.FormatFromMIME (m_FileType.c_str ());
 		if (pOutFormat != NULL) {
@@ -397,8 +394,6 @@ void Document::ExportOB () const
 				BondList.clear ();
 			}
 		}
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
 		gsize nb = ofs.str ().size (), n = 0;
 		while (n < nb) {
 			n += g_output_stream_write (output, ofs.str ().c_str () + n, nb - n, NULL, &error);
@@ -605,14 +600,8 @@ void Document::Save () const
 	if (!m_filename || !m_bWriteable || m_bReadOnly)
 		return;
 	xmlDocPtr xml = NULL;
-	char *old_num_locale, *old_time_locale;
 	const_cast <Document *> (this)->m_SavedResidues.clear ();
 	
-	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-	old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
-	setlocale (LC_TIME, "C");
-
 	try {
 		if (m_pApp && m_pApp->Save (m_filename, m_FileType.c_str (), this, ContentType2D))
 			return;
@@ -667,10 +656,6 @@ void Document::Save () const
 		xml = NULL;
 //		Error (SAVE);
 	}
-	setlocale (LC_NUMERIC, old_num_locale);
-	g_free (old_num_locale);
-	setlocale (LC_TIME, old_time_locale);
-	g_free (old_time_locale);
 	const_cast <Document *> (this)->m_SavedResidues.clear ();
 }
 
@@ -692,8 +677,6 @@ bool Document::Load (xmlNodePtr root)
 		g_free(m_comment);
 		m_comment = NULL;
 	}
-	g_date_clear (&CreationDate, 1);
-	g_date_clear (&RevisionDate, 1);
 	xmlNodePtr node, child;
 	char* tmp;
 	Object* pObject;
@@ -702,18 +685,9 @@ bool Document::Load (xmlNodePtr root)
 		SetId (tmp);
 		xmlFree (tmp);
 	}
-	tmp = (char*) xmlGetProp (root, (xmlChar*) "creation");
-	if (tmp) {
-		g_date_set_parse(&CreationDate, tmp);
-		if (!g_date_valid(&CreationDate)) g_date_clear(&CreationDate, 1);
-		xmlFree (tmp);
-	}
-	tmp = (char*) xmlGetProp (root, (xmlChar*) "revision");
-	if (tmp) {
-		g_date_set_parse(&RevisionDate, tmp);
-		if (!g_date_valid(&RevisionDate)) g_date_clear(&RevisionDate, 1);
-		xmlFree (tmp);
-	}
+
+	gcu::ReadDate (root, "creation", &CreationDate);
+	gcu::ReadDate (root, "revision", &RevisionDate);
 
 	node = GetNodeByName (root, "generator");
 	if (node) {
@@ -826,11 +800,8 @@ xmlDocPtr Document::BuildXMLTree () const
 	if (!g_date_valid (&CreationDate))
 		g_date_set_time_t (&const_cast <Document *> (this)->CreationDate, time (NULL));
 	g_date_set_time_t (&const_cast <Document *> (this)->RevisionDate, time (NULL));
-	gchar buf[64];
-	g_date_strftime (buf, sizeof (buf), "%m/%d/%Y", &CreationDate);
-	xmlNewProp (xml->children, (xmlChar*) "creation", (xmlChar*) buf);
-	g_date_strftime (buf, sizeof (buf), "%m/%d/%Y", &RevisionDate);
-	xmlNewProp (xml->children, (xmlChar*) "revision", (xmlChar*) buf);
+	gcu::WriteDate (xml->children, "creation", &CreationDate);
+	gcu::WriteDate (xml->children, "revision", &RevisionDate);
 	node = xmlNewDocNode (xml, NULL, (xmlChar*)"generator", (xmlChar*)"GChemPaint "VERSION);
 	if (node)
 		xmlAddChild (xml->children, node);
@@ -998,6 +969,8 @@ void Document::RemoveBond (Bond* pBond)
 		if ((pAtom1->GetZ () == 6) && (pAtom1->GetBondsNumber () == 0))
 			m_pView->Update (pAtom1);
 		Parent->Lock (false);
+		if (!m_bUndoRedo && !m_bIsLoading && dynamic_cast <ModifyOperation *> (m_pCurOp))
+			m_pCurOp->AddObject (pMol, 1);
 	}
 	m_DirtyObjects.erase (pBond);
 	delete pBond;
