@@ -55,6 +55,8 @@ struct _GcpFontSel {
 	PangoStretch Stretch;
 	PangoVariant Variant;
 	int Size;
+	bool AllowSlanted;
+	char *LabelText;
 };
 
 typedef struct {
@@ -93,7 +95,9 @@ enum {
 	FONT_SEL_PROP_WEIGHT,
 	FONT_SEL_PROP_STRETCH,
 	FONT_SEL_PROP_VARIANT,
-	FONT_SEL_PROP_SIZE
+	FONT_SEL_PROP_SIZE,
+	FONT_SEL_PROP_ALLOW_SLANTED,
+	FONT_SEL_PROP_LABEL
 };
 
 enum {
@@ -102,6 +106,8 @@ enum {
 };
 static gulong gcp_font_sel_signals [LAST_SIGNAL] = { 0, };
 
+static GObjectClass *font_sel_parent_class;
+
 static void gcp_font_sel_set_label (GcpFontSel *fs)
 {
 	PangoFontDescription *fd = pango_font_description_new ();
@@ -112,7 +118,7 @@ static void gcp_font_sel_set_label (GcpFontSel *fs)
 	pango_font_description_set_stretch (fd, fs->Stretch);
 	pango_font_description_set_size (fd, fs->Size);
 	char *name = pango_font_description_to_string (fd);
-	char *markup = g_markup_printf_escaped ("<span font_desc=\"%s\">%s</span>", name, name);
+	char *markup = g_markup_printf_escaped ("<span font_desc=\"%s\">%s</span>", name, fs->LabelText? fs->LabelText: name);
 	gtk_label_set_markup (fs->Label, markup);
 	g_free (name);
 	g_free (markup);
@@ -190,6 +196,12 @@ gcp_font_sel_get_property (GObject *obj, guint param_id,
 	case FONT_SEL_PROP_SIZE:
 		g_value_set_int (value, fs->Size);
 		break;
+	case FONT_SEL_PROP_ALLOW_SLANTED:
+		g_value_set_int (value, fs->AllowSlanted);
+		break;
+	case FONT_SEL_PROP_LABEL:
+		g_value_set_string (value, fs->LabelText? fs->LabelText: "");
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return;
 	}
@@ -240,15 +252,14 @@ gcp_font_sel_set_property (GObject *obj, guint param_id,
 
 	switch (param_id) {
 	case FONT_SEL_PROP_FAMILY:
-		if (fs->FamilyName)
-			g_free (fs->FamilyName);
+		g_free (fs->FamilyName);
 		fs->FamilyName = g_strdup (g_value_get_string (value));
 		GtkTreeIter iter;
 		char *buf;
 		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (fs->FamilyList), &iter);
 		do {
 			gtk_tree_model_get (GTK_TREE_MODEL (fs->FamilyList), &iter, 0, &buf, -1);
-			if (!strcmp (fs->FamilyName, buf)) {
+			if (!strcmp (fs->FamilyName, buf)) {				
 				GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (fs->FamilyList), &iter);
 				gtk_tree_view_set_cursor (fs->FamilyTree, path, NULL, FALSE);
 				gtk_tree_view_scroll_to_cell (fs->FamilyTree, path, NULL, FALSE, 0., 0.);
@@ -272,20 +283,68 @@ gcp_font_sel_set_property (GObject *obj, guint param_id,
 		fs->Size = g_value_get_int (value);
 		gcp_font_sel_set_size_full (fs, true);
 		return;
+	case FONT_SEL_PROP_ALLOW_SLANTED:
+		fs->AllowSlanted = g_value_get_boolean (value);
+		if (!fs->AllowSlanted) {
+			// remove fonts that are always slanted
+			GtkTreeIter iter;
+			GtkTreeModel *model = GTK_TREE_MODEL (fs->FamilyList);
+			char *family_name;
+			PangoFontFamily *family;
+			PangoFontFace **faces;
+			PangoFontDescription *desc;
+			PangoStyle style;
+			int i, nb;
+			bool valid = gtk_tree_model_get_iter_first (model, &iter);
+			while (valid) {
+				gtk_tree_model_get (model, &iter, 0, &family_name, -1);
+				// get the faces
+				family = fs->Families[family_name];
+				pango_font_family_list_faces (family, &faces, &nb);
+				for (i = 0; i < nb; i++) {
+					desc = pango_font_face_describe (faces[i]);
+					style = pango_font_description_get_style (desc);
+					pango_font_description_free (desc);
+					if (style == PANGO_STYLE_NORMAL)
+						break;
+				}
+				valid = (i == nb)?
+						gtk_list_store_remove (fs->FamilyList, &iter):
+						gtk_tree_model_iter_next (model, &iter);
+				g_free (family_name);
+			}
+		}
+		break;
+	case FONT_SEL_PROP_LABEL: {
+		char const *lbl = g_value_get_string (value);
+		g_free (fs->LabelText);
+		fs->LabelText = (lbl && *lbl)? g_strdup (lbl): NULL;
+		return;
+	}
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return;
 	}
 	select_best_font_face (fs);
 }
 
+static void
+gcp_font_sel_finalize (GObject *obj)
+{
+	GcpFontSel *fs = GCP_FONT_SEL (obj);
+	g_free (fs->LabelText);
+	font_sel_parent_class->finalize (obj);
+}
+
 static void
 gcp_font_sel_class_init (GcpFontSelClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass*) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+	font_sel_parent_class = (GObjectClass*) g_type_class_peek_parent (klass);
 
 	object_class->get_property = gcp_font_sel_get_property;
 	object_class->set_property = gcp_font_sel_set_property;
+	object_class->finalize = gcp_font_sel_finalize;
 
 	widget_class->size_request = gcp_font_sel_size_request;
 	widget_class->size_allocate = gcp_font_sel_size_allocate;
@@ -318,6 +377,14 @@ gcp_font_sel_class_init (GcpFontSelClass *klass)
 			_("The font size (in pango units)"),
 			0, G_MAXINT, 12 * PANGO_SCALE,
 			(GParamFlags) G_PARAM_READWRITE));
+	g_object_class_install_property (object_class, FONT_SEL_PROP_ALLOW_SLANTED,
+		g_param_spec_boolean ("allow-slanted", _("Allow slanted fonts"),
+			_("Whether to allow slanted fonts"),
+			TRUE, (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
+	g_object_class_install_property (object_class, FONT_SEL_PROP_LABEL,
+		g_param_spec_string ("label", _("Label"),
+			_("Preview label"),
+			"", (GParamFlags) G_PARAM_READWRITE));
 
 	gcp_font_sel_signals [CHANGED] = g_signal_new ("changed",
 		G_TYPE_FROM_CLASS (klass),
@@ -339,7 +406,7 @@ static void on_select_family (GtkTreeSelection *selection, GcpFontSel *fs)
 	gtk_tree_model_get (model, &iter, 0, &fs->FamilyName, -1);
 	PangoFontFamily *family = fs->Families[fs->FamilyName];
 	PangoFontFace **faces;
-	int i, nb;
+	int i, besti, nb;
 	g_signal_handler_block (fs->FaceSel, fs->FaceSignal);
 	pango_font_family_list_faces (family, &faces, &nb);
 	gtk_list_store_clear (fs->FaceList);
@@ -359,24 +426,26 @@ static void on_select_family (GtkTreeSelection *selection, GcpFontSel *fs)
 	for (i = 0; i < nb; i++) {
 		name = pango_font_face_get_face_name (faces[i]);
 		desc = pango_font_face_describe (faces[i]);
-		fs->Faces[name] = (PangoFontFace*) g_object_ref (faces[i]);
-		gtk_list_store_append (fs->FaceList, &iter);
-		gtk_list_store_set (fs->FaceList, &iter,
-				  0, name,
-				  -1);
 		// Try to select the best available face
 		Style = pango_font_description_get_style (desc);
-		Weight = pango_font_description_get_weight (desc);
-		Variant = pango_font_description_get_variant (desc);
-		Stretch = pango_font_description_get_stretch (desc);
-		distance = abs (Weight - fs->Weight)
-						+ abs ((Style? Style + 2: 0) - (fs->Style? fs->Style + 2: 0)) * 1000
-						+ abs (Variant - fs->Variant) * 10 + abs (Stretch - fs->Stretch);
-		if (distance < best) {
-			best = distance;
-			selected = iter;
+		if (fs->AllowSlanted || Style == PANGO_STYLE_NORMAL) {
+			fs->Faces[name] = (PangoFontFace*) g_object_ref (faces[i]);
+			gtk_list_store_append (fs->FaceList, &iter);
+			gtk_list_store_set (fs->FaceList, &iter,
+					  0, name,
+					  -1);
+			Weight = pango_font_description_get_weight (desc);
+			Variant = pango_font_description_get_variant (desc);
+			Stretch = pango_font_description_get_stretch (desc);
+			distance = abs (Weight - fs->Weight)
+							+ abs ((Style? Style + 2: 0) - (fs->Style? fs->Style + 2: 0)) * 1000
+							+ abs (Variant - fs->Variant) * 10 + abs (Stretch - fs->Stretch);
+			if (distance < best) {
+				best = distance;
+				selected = iter;
+				besti = i;
+			}
 		}
-		// TODO: write this code
 		pango_font_description_free (desc);
 	}
 	g_free (faces);
@@ -547,6 +616,7 @@ gcp_font_sel_init (GcpFontSel *fs)
 			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
 	g_free (families);
+	fs->AllowSlanted = true;
 }
 
 GSF_CLASS (GcpFontSel, gcp_font_sel,
