@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment-residue.cc 
  *
- * Copyright (C) 2008-2011 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -70,7 +70,7 @@ xmlNodePtr FragmentResidue::Save (xmlDocPtr xml) const
 	xmlNodePtr node = FragmentAtom::Save (xml);
 	if (node) {
 		xmlNodeSetName (node, (xmlChar const *) "residue");
-		static_cast <Document *> (GetFragment ()->GetDocument ())->SaveResidue (m_Residue, node);
+		static_cast <Document const*> (GetFragment ()->GetDocument ())->SaveResidue (m_Residue, node);
 	}
 	return node;
 }
@@ -90,10 +90,12 @@ bool FragmentResidue::Load (xmlNodePtr node)
 	if (!buf || !strlen (buf))
 		return false;
 	Residue *residue = (Residue*) Residue::GetResidue (buf, NULL);
+	Document *doc = static_cast <Document *> (GetDocument ());
+	gcu::Application *app = doc->GetApp ();
 	if (residue) {
 		if (child->next) {
-			Residue *res0 = new Residue (NULL, NULL, NULL, static_cast <Document *> (GetDocument ()));
-			res0->Load (node, false);
+			Residue *res0 = new Residue (NULL, NULL, NULL, doc);
+			res0->Load (node, false, app);
 			if (*residue == *(res0->GetMolecule ()))
 				delete res0; // OK, same molecule
 			else {
@@ -102,7 +104,7 @@ bool FragmentResidue::Load (xmlNodePtr node)
 		}
 	} else if (child->next) {
 		residue = new Residue ();
-		residue->Load (node, false);
+		residue->Load (node, false, app);
 		residue->Register ();
 		// TODO: append residue to the local residues ( or to the document?)
 	} else
