@@ -35,7 +35,6 @@
 #include <glib/gi18n-lib.h>
 #include <unistd.h>
 #include <openbabel/obconversion.h>
-#include <clocale>
 #include <cmath>
 #include <cstring>
 #include <map>
@@ -90,7 +89,7 @@ Molecule::Molecule (TypeId Type): gcu::Molecule (Type)
 	m_IsResidue = false;
 }
 
-Molecule::Molecule (Atom* pAtom): gcu::Molecule  (pAtom)
+Molecule::Molecule (Atom* pAtom): gcu::Molecule (pAtom)
 {
 	m_Alignment = NULL;
 	m_Changed = true;
@@ -417,7 +416,7 @@ bool Molecule::Load (xmlNodePtr node)
 		child = GetNextNodeByName (child->next, "bond");
 		CheckCrossings ((Bond*) pObject);
 	}
-	if (!m_Atoms.empty ()) {
+/*	if (!m_Atoms.empty ()) {
 		Atom* pAtom =  reinterpret_cast <Atom *> (m_Atoms.front ());
 		list<gcu::Atom*>::iterator i = m_Atoms.begin ();
 		i++;
@@ -429,7 +428,7 @@ bool Molecule::Load (xmlNodePtr node)
 			(*j)->RemoveAllCycles ();
 		Chain* pChain = new Chain (this, pAtom); //will find the cycles
 		delete pChain;
-	}
+	}*/
 	buf = (char*) xmlGetProp (node, (const xmlChar*) "valign");
 	if (buf) {
 		m_Alignment = GetDescendant (buf);
@@ -438,6 +437,7 @@ bool Molecule::Load (xmlNodePtr node)
 			return false;
 	}
 	m_Changed = true;
+	pDoc->ObjectLoaded (this);
 	return true;
 }
 
@@ -562,16 +562,11 @@ void Molecule::ExportToGhemical ()
 	BuildOBMol (Mol);
 	char *tmpname = g_strdup ("/tmp/2gprXXXXXX");
 	int f = g_mkstemp (tmpname);
-	gchar *old_num_locale;
 	close (f);
 	ofstream ofs;
 	ofs.open(tmpname);
 	if (!ofs) throw (int) 1;
-	old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-	setlocale(LC_NUMERIC, "C");
 	Conv.Write (&Mol, &ofs);
-	setlocale(LC_NUMERIC, old_num_locale);
-	g_free(old_num_locale);
 	ofs.close();
 	char *command_line = g_strconcat ("ghemical -f ", tmpname, NULL);	
 	g_free (tmpname);
@@ -721,11 +716,7 @@ void Molecule::BuildInChI ()
 		Conv.SetInAndOutFormats (pMolFormat, pInChIFormat);
 		Conv.SetOptions ("xt", OpenBabel::OBConversion::OUTOPTIONS);
 		ostringstream ofs;
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
 		Conv.Write (&Mol, &ofs);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
 		// remove "INCHI=" and the new line char at the end
 		m_InChI = ofs.str ().substr (0, ofs.str ().length () - 2);
 	} else {
@@ -735,10 +726,7 @@ void Molecule::BuildInChI ()
 		close (f);
 		ofstream ofs;
 		ofs.open (tmpname);
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
 		Conv.Write (&Mol, &ofs);
-		setlocale (LC_NUMERIC, old_num_locale);
 		ofs.close ();
 		// calling main_inchi -STDIO -AuxNone -NoLabels
 		char *command = g_strdup_printf ("main_inchi %s -STDIO -AuxNone -NoLabels", tmpname);
@@ -753,7 +741,6 @@ void Molecule::BuildInChI ()
 		if (errors)
 			g_free (errors);
 		g_free (command);
-		g_free (old_num_locale);
 		remove (tmpname);
 		g_free (tmpname);
 	}
@@ -768,12 +755,8 @@ void Molecule::BuildSMILES ()
 	Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
 	BuildOBMol2D (Mol);
 	ostringstream ofs;
-	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
 	Conv.Write (&Mol, &ofs);
-	setlocale (LC_NUMERIC, old_num_locale);
 	//TODO: do something with the string
-	g_free (old_num_locale);
 	string str = ofs.str ().substr (0, ofs.str ().length () - 2);
 	new StringDlg (reinterpret_cast<Document *>(GetDocument ()), str, StringDlg::SMILES);
 }
