@@ -111,7 +111,11 @@ not anymore relevant.
 */
 	void RemoveArrow (ReactionArrow *arrow);
 
+/*!
+@return the localized object generic name.
+*/
 	std::string Name ();
+	void OnLoaded ();
 
 private:
 	void CleanChildren ();
