@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/application.h
  *
- * Copyright (C) 2005-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,6 +27,7 @@
 
 #include "dialog-owner.h"
 #include "structs.h"
+#include "object.h"
 #include <list>
 #include <map>
 #include <set>
@@ -39,11 +40,28 @@ namespace gcu {
 class Document;
 class Dialog;
 struct option_data;
+class TypeDesc;
+class CmdContext;
 
+/*!
+Window states
+*/
 typedef enum {
+/*!
+Normal window.
+*/
 	NormalWindowState,
+/*!
+Maximized window.
+*/
 	MaximizedWindowState,
+/*!
+Minimized window.
+*/
 	MinimizedWindowState,
+/*!
+Full screen window.
+*/
 	FullScreenWindowState
 } WindowState;
 
@@ -56,6 +74,7 @@ class Application: virtual public DialogOwner
 {
 friend class Document;
 friend class Dialog;
+friend class ApplicationPrivate;
 public:
 /*!
 @param name the name of the application.
@@ -68,7 +87,7 @@ the help_name or name parameters will be used.
 The datadir variable is used to build the full path to the help file:
 "file://"+datadir+"/gnome/help/"+name+"/"+LANG+"/"+name".xml".
 */
-	Application (std::string name, std::string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL);
+	Application (std::string name, std::string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL, CmdContext *cc = NULL);
 /*!
 The destructor.
 */
@@ -226,6 +245,10 @@ Method used to retrieve the base configuration node.
 */
 	static GOConfNode *GetConfDir ();
 
+/*!
+Retrieves the icon name that should be set to every window owned by the application.
+@return the icon name for the appication.
+*/
 	std::string const &GetIconName () {return IconName;}
 
 /*!
@@ -244,8 +267,118 @@ Adds all registered options to the context. This should be called once
 just after creating the application and before parsing options.
 */
 	void AddOptions (GOptionContext *context);
+/*!
+@return the default WindowState for the application. New window should use this setting.
+*/
 	static WindowState GetDefaultWindowState () {return DefaultWindowState;}
+/*!
+@return a dummy Application instance which might be used when there is no other
+Application available.
+*/
 	static Application *GetDefaultApplication ();
+	static Application *GetApplication (char const *name);
+	static Application *GetApplication (std::string &name);
+
+	// Object creation related methods
+/*!
+@param TypeName the name of the new type.
+@param CreateFunc a pointer to a function returning a pointer to a new object of the new type.
+@param id the Id of the type to create if a standard one or OtherType for a new type. In this last case, this parameter
+can be omitted.
+
+This method is used to register a new type derived from Object.
+@return the Id of the new type.
+*/
+	TypeId AddType (std::string TypeName, Object* (*CreateFunc) (), TypeId id = OtherType);
+
+/*!
+@param TypeName the name of the new type.
+@param parent the parent of the newly created object or NULL. if NULL, the parameter can be omitted.
+
+Used to create an object of type name TypeName. The AddType() method must have been called with the same
+TypeName parameter. if parent is given and not NULL, the new Object will be a child of parent.
+It will also be given a default Id.
+
+@return a pointer to the newly created Object or NULL if the Object could not be created.
+*/
+	Object* CreateObject (const std::string& TypeName, Object* parent = NULL);
+
+/*!
+@param UIManager the GtkUIManager to populate.
+@param object the Object on which occured the mouse click.
+@param x x coordinate of the mouse click.
+@param y y coordinate of the mouse click.
+
+This method is called to build a contextual menu for the object. It is called by Object::BuildContextualMenu, so
+it should not be necessary to call it directly.
+@return true if something is added to the UIManager, false otherwise.
+*/
+	bool BuildObjectContextualMenu (Object *target, GtkUIManager *UIManager, Object *object, double x, double y);
+/*!
+@param type1 the TypeId of the first class in the rule
+@param rule the new rule value
+@param type2 the TypeId of the second class in the rule
+
+Adds a rule.
+*/
+	void AddRule (TypeId type1, RuleId rule, TypeId type2);
+/*!
+@param type1 the name of the first class in the rule
+@param rule the new rule value
+@param type2 the name of the second class in the rule
+
+Adds a rule.
+*/
+	void AddRule (const std::string& type1, RuleId rule, const std::string& type2);
+/*!
+@param type the TypeId of a class
+@param rule a RuleId value
+
+@return the set of rules correponding to the RuleId value for this class.
+*/
+	 const std::set<TypeId>& GetRules (TypeId type, RuleId rule);
+
+/*!
+@param type the name of a class
+@param rule a RuleId value
+
+@return the set of rules correponding to the RuleId value for this class.
+*/
+	const std::set<TypeId>& GetRules (const std::string& type, RuleId rule);
+
+/*!
+@param Id the TypeId of a class
+@param Label the string to display in a contextual menu
+
+Used to give a label for contextual menus used when the creation of an instance of
+the class seems possible.
+*/
+	void SetCreationLabel (TypeId Id, std::string Label);
+
+/*!
+@param Id the TypeId of a class
+
+@return the string defined by SetCreationLabel.
+*/
+	const std::string& GetCreationLabel (TypeId Id);
+
+/*!
+@param TypeName the name of a class
+
+@return the string defined by SetCreationLabel.
+*/
+	const std::string& GetCreationLabel (const std::string& TypeName);
+
+/*!
+@param Id the TypeId of the Object derived class
+@param cb the BuildMenuCb callback to call when building the menu.
+
+adds a callback for modifying the contextual menu of objects of type Id.
+*/
+	void AddMenuCallback (TypeId Id, BuildMenuCb cb);
+
+	TypeDesc const *GetTypeDescription (TypeId Id);
+	CmdContext *GetCmdContext ();
 
 protected:
 
@@ -260,9 +393,6 @@ private:
 	void AddDocument (Document *Doc) {m_Docs.insert (Doc);}
 	void RemoveDocument (Document *Doc);
 
-public:
-	static WindowState DefaultWindowState;
-
 private:
 	std::string Name;
 	std::string HelpName;
@@ -272,12 +402,16 @@ private:
 	std::string IconName;
 	static GOConfNode *m_ConfDir;
 	std::list<option_data> m_Options;
+	static WindowState DefaultWindowState;
+	std::map<TypeId, TypeDesc> m_Types;
 
 protected:
 /*!
 std::map of the supported pixbuf formats. Keys are the mime type names.
 */
-	std::map<std::string, GdkPixbufFormat*> m_SupportedPixbufFormats;	
+	std::map<std::string, GdkPixbufFormat*> m_SupportedPixbufFormats;
+
+	CmdContext *m_CmdContext;
 
 /*!\var m_Docs
 The currently opened documents.
