@@ -2,7 +2,7 @@
  * Gnome Chemistry Utils
  * cmd-context.h 
  *
- * Copyright (C) 2007-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,27 +23,43 @@
 #ifndef GCU_CMD_CONTEXT_H
 #define GCU_CMD_CONTEXT_H
 
+#include "macros.h"
 #include <goffice/goffice.h>
 
-G_BEGIN_DECLS
+namespace gcu {
 
-/*!\file
-Declaration of the GcuCmdContext structure, a GObject implementing the
-GOCmdContext interface from goffice. This is mandatory to be able to edit
-charts.
-*/
+class Application;
 
-/*!
-@return the GType associated to GcuCmdContext.
-*/
-#define GCU_TYPE_CMD_CONTEXT		(gcu_cmd_context_get_type ())
-GType		gcu_cmd_context_get_type   (void);
+class CmdContext {
+public:
+	friend class Application;
+	CmdContext (Application *App);
+	virtual ~CmdContext ();
 
-/*!
-@return a static global GcuCmdContext instance.
-*/
-GOCmdContext *gcu_get_cmd_context (void);
+	typedef enum {
+		ResponseDefault,
+		ResponseOK = 1 << 0,
+		ResponseCancel = 1 << 1,
+		ResponseYes = 1 << 2,
+		ResponseNo = 1 << 3,
+		ResponseClose = 1 << 4
+	} Response;
 
-G_END_DECLS
+	typedef enum {
+		SeverityMessage,
+		SeverityWarning,
+		SeverityError
+	} Severity;
 
-#endif	/* GCU_CMD_CONTEXT_H */
+	virtual Response GetResponse (char const *message, int responses) = 0;
+	virtual void Message (char const *message, Severity severity, bool modal) = 0;
+
+	GOIOContext *GetNewGOIOContext ();
+
+GCU_PROT_PROP (GOCmdContext *, GOCmdContext)
+GCU_PROT_PROP (Application *, App)
+};
+
+}	//	namespace gcu
+
+#endif //	GCU_CMD_CONTEXT_H
