@@ -51,7 +51,7 @@ Document::~Document ()
 		m_App->RemoveDocument (this);
 }
 
-gchar* Document::GetNewId (gchar* id, bool Cache)
+char* Document::GetNewId (char const *id, bool Cache)
 {
 	gchar *Id = g_strdup (id);
 	int i = 0;
@@ -66,26 +66,22 @@ gchar* Document::GetNewId (gchar* id, bool Cache)
 	if (s.size ())
 		j = atoi (s.c_str ());
 	char* key = g_strdup (buf);
-	while (snprintf (buf + i, 16, "%d", j), GetDescendant (buf) != NULL)
-		j++;
+	while (snprintf (buf + i, 16, "%d", j++), GetDescendant (buf) != NULL) ;
 	Id = g_strdup_printf ("%d", j);
-	Object *obj = GetDescendant (id);
-	if (obj && m_DirtyObjects.find (obj) == m_DirtyObjects.end ()) { //ensure the existing object is not new
-		if (Cache) {
-			m_TranslationTable[key] = Id;
-			m_TranslationTable[id] = buf;
-		}
-		if (m_PendingTable.size () > 0) {
-			std::map <std::string, list <PendingTarget> >::iterator it, end = m_PendingTable.end ();
-			if ((it = m_PendingTable.find (id)) != end && m_PendingTable.find (buf) == end) {
-				// Hmm, this might be unsecure if several imported objects have the same Id
-				m_PendingTable[buf] = (*it).second;
-				m_PendingTable.erase (it);
-			}
-		}
+	if (Cache) {
+		m_TranslationTable[key] = Id;
+		m_TranslationTable[id] = buf;
 	}
 	g_free (Id);
 	g_free (key);
+	if (m_PendingTable.size () > 0) {
+		std::map <std::string, list <PendingTarget> >::iterator it, end = m_PendingTable.end ();
+		if ((it = m_PendingTable.find (id)) != end) {
+			// Hmm, this might be unsecure if several imported objects have the same Id
+			m_PendingTable[buf] = (*it).second;
+			m_PendingTable.erase (it);
+		}
+	}
 	return buf;
 }
 
@@ -99,17 +95,15 @@ Residue const *Document::GetResidue (char const *symbol, bool *ambiguous)
 	return Residue::GetResidue (symbol, ambiguous);
 }
 
-bool Document::SetTarget (char const *id, Object **target, Object *parent, Object *owner) throw (std::runtime_error)
+bool Document::SetTarget (char const *id, Object **target, Object *parent, Object *owner, Action action) throw (std::runtime_error)
 {
 	if (target == NULL)
 	    throw std::runtime_error ("Can't set a NULL target.");
-	*target = parent->GetDescendant (id);
-	if (*target)
-		return true;
 	PendingTarget pt;
 	pt.target = target;
 	pt.parent = parent;
 	pt.owner = owner;
+	pt.action = action;
 	m_PendingTable[id].push_back (pt);
 	return false;
 }
@@ -118,30 +112,60 @@ bool Document::Loaded () throw (LoaderError)
 {
 	unsigned count = 0;
 	std::map <std::string, list <PendingTarget> >::iterator i, end = m_PendingTable.end ();
+	std::set <Object *> Deleted;
+	std::set <Object *>::iterator new_end = m_NewObjects.end ();
 	for (i = m_PendingTable.begin (); i != end; i++) {
 		std::string id = (*i).first;
 		std::list <PendingTarget> &l = (*i).second;
 		std::list <PendingTarget>::iterator j = l.begin (), jend = l.end ();
 		Object *obj = (*j).parent->GetDescendant (id.c_str ());
+		if (obj == NULL)
+			obj = (*j).parent->GetDocument ()->GetDescendant (id.c_str ());
+		if (obj && m_NewObjects.find (obj) == new_end)
+			obj = NULL;
 		if (obj == NULL) {
-			m_PendingTable.clear ();
-			std::ostringstream str;
-			// Note to translators: the two strings are concatenated with the missing id between them.
-			str << _("The input contains a reference to object \"") << id << _("\" but no object with this Id is described.");
-			throw LoaderError (str.str ());
+			switch ((*j).action) {
+			case ActionException: {
+				m_PendingTable.clear ();
+				std::ostringstream str;
+				// Note to translators: the two strings are concatenated with the missing id between them.
+				str << _("The input contains a reference to object \"") << id << _("\" but no object with this Id is described.");
+				throw LoaderError (str.str ());
+			}
+			case ActionDelete:
+				if ((*j).owner) {
+					Deleted.insert ((*j).owner);
+					delete (*j).owner;
+					(*j).owner = NULL;
+				}
+			case ActionIgnore:
+				break;
+			}
 		} else while (j != jend) {
-			*(*j).target = obj;
-			if ((*j).owner)
-				(*j).owner->OnLoaded ();
-			count++;
-			j++;
+			std::set <Object *>::iterator d = Deleted.end ();
+			if (Deleted.find ((*j).owner) == d) {
+				*(*j).target = obj;
+				if ((*j).owner)
+					(*j).owner->OnLoaded ();
+				count++;
+			}
+				j++;
 		}
 	}
 	m_PendingTable.clear ();
-	m_DirtyObjects.clear ();
+	m_NewObjects.clear ();
+	// call OnLoaded for dirty objects
+	std::set <Object *>::iterator k, kend = m_DirtyObjects.end ();
+	for (k = m_DirtyObjects.begin (); k != kend; k++)
+		(*k)->OnLoaded ();
 	return count > 0;
 }
 
+void Document::ObjectLoaded (Object *obj)
+{
+	m_NewObjects.insert (obj);
+}
+
 std::string Document::Name ()
 {
 	return _("Document");
