@@ -222,6 +222,77 @@ The name of the structure type (see http://www.iucr.org/__data/iucr/cifdic_html/
 The name of the space group for a crystal. The Hall name is used as it is unique.
 */
 	GCU_PROP_SPACE_GROUP,
+/******************************************************************************
+ * Spectrum related properties
+ ******************************************************************************/
+/*!
+The spectrum type, acceptable values are:
+	"INFRARED SPECTRUM",
+	"RAMAN SPECTRUM",
+	"INFRARED PEAK TABLE",
+	"INFRARED INTERFEROGRAM",
+	"INFRARED TRANSFORMED SPECTRUM",
+	"UV-VISIBLE SPECTRUM",
+	"NMR SPECTRUM",
+	"NMR FID",
+	"NMR PEAK TABLE",
+	"NMR PEAK ASSIGNMENTS",
+	"MASS SPECTRUM",
+	"UV-VIS SPECTRUM",
+	"UV/VISIBLE SPECTRUM",
+	"UV/VIS SPECTRUM".
+*/
+	GCU_PROP_SPECTRUM_TYPE,
+/*!
+The data number of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_NPOINTS,
+/*!
+The x data of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_DATA_X,
+/*!
+The y data of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_DATA_Y,
+/*!
+The real components of a spectrum made of complex data.
+*/
+	GCU_PROP_SPECTRUM_DATA_REAL,
+/*!
+The imaginary components of a spectrum made of complex data.
+*/
+	GCU_PROP_SPECTRUM_DATA_IMAGINARY,
+/*!
+The x data unit for a spectrum, acceptable values are:
+	"1/CM",
+	"TRANSMITTANCE",
+	"ABSORBANCE",
+	"PPM",
+	"NANOMETERS",
+	"MICROMETERS",
+	"SECONDS",
+	"HZ",
+	"M/Z",
+	"RELATIVE ABUNDANCE".
+*/
+	GCU_PROP_SPECTRUM_X_UNIT,
+/*!
+The lowest x datum of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_X_MIN,
+/*!
+The largest x datum of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_X_MAX,
+/*!
+The offset x datum of a spectrum.
+*/
+	GCU_PROP_SPECTRUM_X_OFFSET,
+/*!
+The NMR spectrometer frequency.
+*/
+	GCU_PROP_SPECTRUM_NMR_FREQ,
 /*!
 The first invalid value. It might be used as an error value.
 */
