@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+#include <gcu/application.h>
 #include <gcu/document.h>
 #include <gcu/loader.h>
 #include <gcu/molecule.h>
@@ -46,6 +47,7 @@ static map<string, unsigned> KnownProps;
 
 typedef struct {
 	Document *doc;
+	Application *app;
 	GOIOContext *context;
 	stack<Object*> cur;
 	ContentType type;
@@ -90,7 +92,8 @@ bool cml_write_atom (G_GNUC_UNUSED CMLLoader *loader, GsfXMLOut *xml, Object *ob
 		double x, y;
 		prop = object->GetProperty (GCU_PROP_POS2D);
 		if (prop.length ()) {
-			sscanf (prop.c_str (), "%lg %lg", &x, &y);
+			istringstream in (prop);
+			in >> x >> y;
 			gsf_xml_out_add_float (xml, "x2", x, -1);
 			gsf_xml_out_add_float (xml, "y2", -y, -1); // reverse y order
 		}
@@ -105,7 +108,8 @@ bool cml_write_atom (G_GNUC_UNUSED CMLLoader *loader, GsfXMLOut *xml, Object *ob
 		double x, y, z;
 		prop = object->GetProperty (GCU_PROP_POS3D);
 		if (prop.length ()) {
-			sscanf (prop.c_str (), "%lg %lg %lg", &x, &y, &z);
+			istringstream in (prop);
+			in >> x >> y >> z;
 			gsf_xml_out_add_float (xml, "x3", x, -1);
 			gsf_xml_out_add_float (xml, "y3", y, -1);
 			gsf_xml_out_add_float (xml, "z3", z, -1);
@@ -266,10 +270,11 @@ cml_scalar_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 	CMLReadState *state = (CMLReadState *) xin->user_state;
 	if (state->proptype == "xsd:double") {
-		double val = strtod (xin->content->str, NULL);
+		double val = g_ascii_strtod (xin->content->str, NULL);
 		if (state->curstr == "units:angstrom" || state->curstr == "")
 			val *= 100.;
-		char *buf = g_strdup_printf ("%g", val);
+		char buf[G_ASCII_DTOSTR_BUF_SIZE];
+		g_ascii_dtostr (buf, G_ASCII_DTOSTR_BUF_SIZE, val);
 		state->doc->SetProperty (state->cur_prop, buf);
 		g_free (buf);
 	} else if (state->proptype == "xsd:string")
@@ -323,7 +328,7 @@ cml_atom_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	CMLReadState *state = (CMLReadState *) xin->user_state;
 	Object *parent = state->cur.top ();
-	Object *obj = Object::CreateObject ("atom", parent? parent: state->doc);
+	Object *obj = state->app->CreateObject ("atom", parent? parent: state->doc);
 	obj->SetProperty (GCU_PROP_ATOM_SYMBOL, "C");
 	map <string, unsigned>::iterator it;
 	if (attrs)
@@ -336,7 +341,7 @@ cml_atom_start (GsfXMLIn *xin, xmlChar const **attrs)
 			}
 			if (!strcmp ((char const *) *attrs, "y2")) {
 				attrs++;
-				double x = -strtod ((char const *) *attrs, NULL); // reverse y direction, see comment in cml_write_atom
+				double x = -g_ascii_strtod ((char const *) *attrs, NULL); // reverse y direction, see comment in cml_write_atom
 				ostringstream res;
 				res << x;
 				obj->SetProperty (GCU_PROP_Y, res.str ().c_str ());
@@ -346,6 +351,7 @@ cml_atom_start (GsfXMLIn *xin, xmlChar const **attrs)
 			}
 			attrs++;
 		}
+	state->doc->ObjectLoaded (obj);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -355,7 +361,7 @@ static void
 cml_bond_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	CMLReadState *state = (CMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("bond", state->cur.top ());
+	Object *obj = state->app->CreateObject ("bond", state->cur.top ());
 	if (obj) {
 		map <string, unsigned>::iterator it;
 		if (attrs)
@@ -375,6 +381,7 @@ cml_bond_start (GsfXMLIn *xin, xmlChar const **attrs)
 			}
 	}
 	state->cur.push (obj);
+	state->doc->ObjectLoaded (obj);
 }
 
 static void
@@ -466,12 +473,13 @@ cml_mol_start (GsfXMLIn *xin, xmlChar const **attrs)
 	GSF_XML_IN_NODE_END
 	};
 	CMLReadState	*state = (CMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("molecule", state->cur.top ());
+	Object *obj = state->app->CreateObject ("molecule", state->cur.top ());
 	state->cur.push (obj);
 	static GsfXMLInDoc *doc = NULL;
 	if (NULL == doc)
 		doc = gsf_xml_in_doc_new (mol_dtd, NULL);
 	gsf_xml_in_push_state (xin, doc, state, (GsfXMLInExtDtor) NULL, attrs);
+	state->doc->ObjectLoaded (obj);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -488,6 +496,7 @@ ContentType CMLLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 	bool  success = false;
 
 	state.doc = doc;
+	state.app = doc->GetApplication ();
 	state.context = io;
 	state.cur.push (doc);
 	state.type = ContentTypeMisc;
