@@ -27,6 +27,7 @@
 #include <gcp/application.h>
 #include <gcp/molecule.h>
 #include "selectiontool.h"
+#include "lassotool.h"
 #include "erasertool.h"
 #include "group.h"
 #include "bracketstool.h"
@@ -42,7 +43,6 @@ static Object* CreateGroup ()
 
 gcpSelectionPlugin::gcpSelectionPlugin(): gcp::Plugin()
 {
-	GroupType = Object::AddType ("group", CreateGroup);
 }
 
 gcpSelectionPlugin::~gcpSelectionPlugin()
@@ -53,10 +53,11 @@ static gcp::IconDesc icon_descs[] = {
 	{"gcp_Selection", gcp_selection_24},
 	{"gcp_Eraser", gcp_eraser_24},
 	{"gcp_Horiz", gcp_horiz_24},
+	{"gcp_Lasso", gcp_lasso_24},
 	{"gcp_Vert", gcp_vert_24},
 	{"gcp_Rotate", gcp_rotate_24},
 	{"gcp_Merge", gcp_merge_24},
-/*	{"gcp_Brackets", gcp_brackets_24},*/
+	{"gcp_Brackets", gcp_brackets_24},
 	{NULL, NULL},
 };
 
@@ -66,10 +67,13 @@ static GtkRadioActionEntry entries[] = {
 		0	},
 	{	"Erase", "gcp_Eraser", N_("Erase"), NULL,
 		N_("Eraser"),
-		0	}/*,
+		0	},
+	{	"Lasso", "gcp_Lasso", N_("Area selection"), NULL,
+		N_("Area selection"),
+		0	},
 	{	"Brackets", "gcp_Brackets", N_("Brackets"), NULL,
 		N_("Brackets"),
-		0	},*/
+		0	},
 };
 
 static const char *ui_description =
@@ -77,8 +81,9 @@ static const char *ui_description =
 "  <toolbar name='SelectToolbar'>"
 "	 <placeholder name='Select1'>"
 "      <toolitem action='Select'/>"
+"      <toolitem action='Lasso'/>"
 "      <toolitem action='Erase'/>"
-//"      <toolitem action='Brackets'/>"
+"      <toolitem action='Brackets'/>"
 "	 </placeholder>"
 "	 <placeholder name='Select2'/>"
 "	 <placeholder name='Select3'/>"
@@ -87,9 +92,11 @@ static const char *ui_description =
 
 void gcpSelectionPlugin::Populate (gcp::Application* App)
 {
+	GroupType = App->AddType ("group", CreateGroup);
 	App->AddActions (entries, G_N_ELEMENTS (entries), ui_description, icon_descs);
 	App->RegisterToolbar ("SelectToolbar", 0);
 	new gcpSelectionTool (App);
+	new gcpLassoTool (App);
 	new gcpEraserTool (App);
 	new gcpBracketsTool (App);
 	App->ActivateTool ("Select", true);
