@@ -4,7 +4,7 @@
  * Gnome Crystal
  * application.cc 
  *
- * Copyright (C) 2001-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -47,9 +47,9 @@ static Object *CreateAtom ()
 
 bool gcApplication::m_bInit = false;
 
-gcApplication::gcApplication(): Application ("gcrystal")
+gcApplication::gcApplication(): gcr::Application ()
 {
-	gcu::Loader::Init ();
+	gcu::Loader::Init (this);
 	m_SupportedMimeTypes.push_back ("application/x-gcrystal");
 	m_WriteableMimeTypes.push_back ("application/x-gcrystal");
 	// browse available loaders
